<?php

// Check if class already exists

//if (!class_exists("WPsite_Email_Signups_FF")):

// AJAX functions for email signups

add_action( 'wp_ajax_wps_ff_add_email', 			array('WPsite_Email_Signups_FF', 'addEmail'));
add_action( 'wp_ajax_nopriv_wps_ff_add_email', 		array('WPsite_Email_Signups_FF', 'addEmail'));

/**
 * WPsite_Email_Signups_FF class.
 *
 * This class is used to record all stats for flyout fires.
 */
class WPsite_Email_Signups_FF {

	/**
	 * table_name
	 *
	 * (default value: 'wpsite_flyout_fire_emails')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	public static $table_name = 'wpsite_flyout_fire_emails';

	/**
	 * Create Table when this class is loaded
	 *
	 * @access public
	 * @return void
	 */
	function __construct() {
		self::createTable();
	}

	/**
	 * Creates the Stats table if it does not exist
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function createTable(){

		global $wpdb;

		$wpdb->query("
			CREATE TABLE IF NOT EXISTS " . self::getTableName() . " (
				`id` int(11) NOT NULL AUTO_INCREMENT,
				`date` date NOT NULL,
				`flyout_fire_id` int(11) NOT NULL,
				`email` varchar(255) NOT NULL UNIQUE,
				`first_name` varchar(50) NOT NULL DEFAULT '',
				`last_name` varchar(50) NOT NULL DEFAULT '',
				PRIMARY KEY (`id`)
			) ENGINE=InnoDB DEFAULT CHARSET=latin1 AUTO_INCREMENT=1;
		");

	}

	/**
	 * Get the emails over a date range in all time
	 *
	 * @access public
	 * @static
	 * @param mixed $start
	 * @param mixed $end
	 * @param mixed $id (default: null)
	 * @return void
	 */
	static function getEmails($start, $end, $email = null, $select = '*') {

		// Safey to make sure the table is there
		self::createTable();

		$query = null;

		global $wpdb;

		// All Flyout Fires, All Time

		if ($start == null && $end == null && $email == null) {
			$query = 'SELECT ' . $select . ' FROM ' . self::getTableName();
		}

		// All Flyout Fires, After Date

		else if ($start != null && $end == null && $email == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` = %s', $start);
		}

		// All Flyout Fires, Date Range

		else if ($start != null && $end != null && $email == null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `date` >= %s AND `date` <= %s', $start, $end);
		}

		// Single Email

		else if ($start == null && $end == null && $email != null) {
			$query = $wpdb->prepare('SELECT ' . $select . ' FROM ' . self::getTableName() . ' WHERE `email` = %d', $email);
		}

		// No query was created

		if (!isset($query)) {
			return false;
		}

		$result = $wpdb->get_results($query, 'ARRAY_A');

		return $result;
	}

	/**
	 * Record an impression made by a flyout fire
	 * This function is called via PHP.
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function addEmail() {

		// No First Name

		if (!isset($_POST['first_name'])) {
			$_POST['first_name'] = '';
		}

		// No Last Name

		if (!isset($_POST['last_name'])) {
			$_POST['last_name'] = '';
		}

		include_once(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/admin/model/db.php');

		$wpsite_flyout_fire_db = new WPsite_Flyout_Fire_DB();
		$flyout_fire = $wpsite_flyout_fire_db->get_data_from_id($_POST['flyout_fire_id']);
		$flyout_fire['news_letter'] = unserialize($flyout_fire['news_letter']);
		$success_mesage = stripcslashes($flyout_fire['news_letter']['success_message']);

		// Add email to WordPress table

		if ($_POST['type'] == 'WordPress') {

			// Could not find Flyout Fire ID

			if (!isset($_POST['flyout_fire_id']) || $_POST['flyout_fire_id'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'Could not find Flyout Fire ID.'
				));
				die();
			}

			// No Email

			if (!isset($_POST['email']) || $_POST['email'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'No Email address provided.'
				));
				die();
			}

			// Invalid Email Address

			if (!filter_var($_POST['email'], FILTER_VALIDATE_EMAIL)) {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'Invalid Email provided.'
				));
				die();
			}

			global $wpdb;
			$today = date(WPSITE_FLYOUT_FIRE_DATE_FORMAT);

			$query = $wpdb->prepare('INSERT IGNORE INTO ' . self::getTableName() . ' (
				`date`,
				`flyout_fire_id`,
				`email`,
				`first_name`,
				`last_name`
				) VALUES (%s, %d, %s, %s, %s)',
				$today,
				$_POST['flyout_fire_id'],
				$_POST['email'],
				$_POST['first_name'],
				$_POST['last_name']
			);

			$result = $wpdb->query($query);

			if ($result) {

				// Add Cookie to let us know not to show this popup ever again.

				setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

				echo json_encode(array(
					'status'		=> 'check',
					'message'		=> $success_mesage,
					'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
				));

			} else {
				echo json_encode(array(
					'status'	=> 'check',
					'message'	=> 'We already have your email!'
				));

			}
		}

		// MailChimp

		else if ($_POST['type'] == 'MailChimp') {

			require(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/includes/mailchimp/MailChimp.php');

			if (!isset($flyout_fire['news_letter']['MailChimp']['api_key']) || $flyout_fire['news_letter']['MailChimp']['api_key'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> __('MailChimp account is not setup properly.'),
				));

				die();
			}

			if (!isset($flyout_fire['news_letter']['MailChimp']['list']) || $flyout_fire['news_letter']['MailChimp']['list'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> __('MailChimp: No list specified.'),
				));

				die();
			}

			if ( !isset($flyout_fire['news_letter']['MailChimp']['optin']) ) {
				$flyout_fire['news_letter']['MailChimp']['optin'] = true;
			}

			$MailChimp = new WPS_MailChimp($flyout_fire['news_letter']['MailChimp']['api_key']);
			$result = $MailChimp->call('lists/subscribe', array(
                'id'                => $flyout_fire['news_letter']['MailChimp']['list'],
                'email'             => array('email'=>$_POST['email']),
                'merge_vars'        => array('FNAME'=>$_POST['first_name'], 'LNAME'=>$_POST['last_name']),
                'double_optin'      => $flyout_fire['news_letter']['MailChimp']['optin'],
                'update_existing'   => true,
                'replace_interests' => false,
                'send_welcome'      => false,
            ));

            if ($result) {

	            if (isset($result['email'])) {

		            // Add Cookie to let us know not to show this popup ever again.

					setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

					echo json_encode(array(
						'status'		=> 'check',
						'message'		=> $success_mesage,
						'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
					));

					die();
	            }

	            else if (isset($result['status']) && $result['status'] == 'error') {
					echo json_encode(array(
						'status'		=> 'warning',
						'message'		=> $result['error'],
					));

					die();
	            }
            } else {

	            echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> __('Unable to subscribe.'),
				));

				die();
            }
		}

		// Add email to aweber

		else if ($_POST['type'] == 'aweber') {

			require(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/includes/aweber/aweber_api.php');

			if (!isset($flyout_fire['news_letter']['aweber']['consumer_key']) || $flyout_fire['news_letter']['aweber']['consumer_key'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> __('Aweber account is not setup properly'),
				));

				die();
			}

			$aweber = new AWeberAPI($flyout_fire['news_letter']['aweber']['consumer_key'], $flyout_fire['news_letter']['aweber']['consumer_secret']);

			try {
				$account = $aweber->getAccount($flyout_fire['news_letter']['aweber']['access_key'], $flyout_fire['news_letter']['aweber']['access_secret']);
				$list = $account->loadFromUrl('/accounts/' . $account->id . '/lists/' . $flyout_fire['news_letter']['aweber']['list_id']);

				$subscriber = array(
					'email' 	=> $_POST['email'],
					'name'		=> $_POST['first_name'] . ' ' . $_POST['last_name'],
					'ip' 		=> $_SERVER['REMOTE_ADDR']
				);

				$newSubscriber = $list->subscribers->create($subscriber);

				// Add Cookie to let us know not to show this popup ever again.

				setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

				echo json_encode(array(
					'status'		=> 'check',
					'message'		=> $success_mesage,
					'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
				));

				die();

			} catch (AWeberAPIException $exc) {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> $exc->message,
				));

				die();
			}
		}

		// Add email to Get Response

		else if ($_POST['type'] == 'getresponse') {

			require(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/includes/getresponse/jsonRPCClient.php');

			$api = new jsonRPCClient('http://api2.getresponse.com');

			try {
				$api->add_contact(
					$flyout_fire['news_letter']['getresponse']['api_key'],
				    array (
				        'campaign'  => $flyout_fire['news_letter']['getresponse']['campaign'],
				        'name'      => $_POST['first_name'] . ' ' . $_POST['last_name'],
				        'email'     => $_POST['email'],
				    )
				);

				// Add Cookie to let us know not to show this flyout fire fire ever again.

				setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

				echo json_encode(array(
					'status'		=> 'check',
					'message'		=> $success_mesage,
					'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
				));

				die();

			} catch (RuntimeException $exc) {

				$msg = $exc->getMessage();
				$msg = substr($msg, 0, strpos($msg, ";"));

				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> $msg,
				));

				die();
			}
		}

		// Add email to Campaign Monitor

		else if ($_POST['type'] == 'campaignmonitor') {

			require(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/includes/campaignmonitor/csrest_subscribers.php');

			$wrap = new CS_REST_Subscribers($flyout_fire['news_letter']['campaignmonitor']['list'], $flyout_fire['news_letter']['campaignmonitor']['api_key']);

			// Check if subscribor is already subscribed

			$result = $wrap->get($_POST['email']);

			if ($result->was_successful()) {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'You are already subscribed to this list.',
				));

				die();
			}

			$result = $wrap->add(array(
				'EmailAddress' 	=> $_POST['email'],
				'Name' 			=> $_POST['first_name'] . ' ' . $_POST['last_name'],
				'Resubscribe' 	=> true
			));

			if ($result->was_successful()) {

			    // Add Cookie to let us know not to show this flyout fire fire ever again.

				setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

				echo json_encode(array(
					'status'		=> 'check',
					'message'		=> $success_mesage,
					'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
				));

				die();

			} else {

				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> $result->response->Message,
				));

				die();
			}
		}

		// Add email to Mad Mimi

		else if ($_POST['type'] == 'madmimi') {

			require(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/includes/madmimi/MadMimi.class.php');

			$mailer = new MadMimi($flyout_fire['news_letter']['madmimi']['username'], $flyout_fire['news_letter']['madmimi']['api_key']);

			// No Email

			if (!isset($_POST['email']) || $_POST['email'] == '') {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'No Email address provided.'
				));
				die();
			}

			// Invalid Email Address

			if (!filter_var($_POST['email'], FILTER_VALIDATE_EMAIL)) {
				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> 'Invalid Email provided.'
				));
				die();
			}

			try {

				// Check if user is already in list

				$result = $mailer->Memberships($_POST['email']);
				$lists  = new SimpleXMLElement($result);

				if ($lists->list) {
					foreach ($lists->list as $l) {
						if ($l->attributes()->{'name'}->{0} == $flyout_fire['news_letter']['madmimi']['list']) {

							echo json_encode(array(
								'status'		=> 'check',
								'message'		=> 'You are already subscribed to this list.',
							));

							die();
						}
					}
			    }

				$result = $mailer->AddMembership($flyout_fire['news_letter']['madmimi']['list'], $_POST['email'], array(
					'first_name'	=> $_POST['first_name'],
					'last_name'		=> $_POST['last_name'],
				));

				// Add Cookie to let us know not to show this flyout fire fire ever again.

				setcookie(WPsite_Flyout_Fire::$prefix . 'email_success_' . $_POST['flyout_fire_id'], true, 0, '/');

				echo json_encode(array(
					'status'		=> 'check',
					'message'		=> $success_mesage,
					'conversion'	=> self::recordConversion($_POST['flyout_fire_id']),
				));

				die();

			} catch (RuntimeException $exc) {

				echo json_encode(array(
					'status'	=> 'warning',
					'message'	=> $msg,
				));

				die();
			}
		}

		die(); // this is required to terminate immediately and return a proper response
	}

	/**
	 * Record a converison made by a newsletter signup
	 *
	 * @access public
	 * @static
	 * @param mixed $flyout_fire_id
	 * @return void
	 */
	static function recordConversion($flyout_fire_id) {

		include_once(WPSITE_FLYOUT_FIRE_PLUGIN_DIR . '/core/model/stats.php');

		// Return if no data is given

		if (!isset($flyout_fire_id)) {
			return 'Flyout Fire conversion was NOT able to be recored because the Flyout Fire ID was not retrieved.';
		}

		// Return if user is admin or higher

		if (current_user_can('edit_published_posts')) {
			return 'Flyout Fire conversion was NOT able to be recored because current user is not authorized to log clicks.';
		}

		// Return false if crawler

		if (WPsite_Stats_FF::isBot()) {
			return 'Flyout Fire conversion was NOT able to be recored because there is no user.  A crawler was detected.';
		}

		global $wpdb;
		$today = date(WPSITE_FLYOUT_FIRE_DATE_FORMAT);

		// Check if entry already exsits

		$query = $wpdb->prepare('SELECT * FROM ' . WPsite_Stats_FF::getTableName() . ' WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $flyout_fire_id);
		$flyout_fire_clicks = $wpdb->query($query);

		// Entry aleady exists, just add 1

		if (isset($flyout_fire_clicks) && $flyout_fire_clicks != 0) {

			$query = $wpdb->prepare('UPDATE ' . WPsite_Stats_FF::getTableName() . ' SET
				`clicks` = `clicks` + 1
				WHERE `date` = %s AND `flyout_fire_id` = %d', $today, $flyout_fire_id);

			$result = $wpdb->query($query);

		}

		// Entry does not exist, create a new one and set impressions to 1

		else {
			$query = $wpdb->prepare('INSERT INTO ' . WPsite_Stats_FF::getTableName() . ' (
				`date`,
				`flyout_fire_id`,
				`impressions`,
				`clicks`
				) VALUES (%s, %d, 0, 1)',
				$today,
				$flyout_fire_id
			);

			$result = $wpdb->query($query);
		}

		if ($result) {
			return 'Flyout Fire conversion was able to be recored.';
		} else {
			return 'Flyout Fire conversion was NOT able to be recored.';
		}

		return '';
	}

	/**
	 * Deletes the stats for a flyout fire when the flyout fire is deleted
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function deleteEmail($email) {

		// Return false if flyout_fire_id is not set

		if (!isset($email)) {
			return false;
		}

		global $wpdb;

		// Check if entry already exsits

		$query = $wpdb->prepare('DELETE FROM ' . self::getTableName() . ' WHERE `email` = %d', $email);
		$result = $wpdb->query($query);

		return $result;

	}

	/**
	 * Returns the proper table name based off of site ID
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function getTableName(){

		global $wpdb;

		return '`' . $wpdb->prefix . self::$table_name . '`';
	}

}

//endif;